@@ -45,7 +45,7 @@ int load_equalizer(mpg123_handle *mh)
 		}
 		else
 		{
-			error1("Can't open equalizer file '%s'.",equalfile);
+			merror("Can't open equalizer file '%s'.",equalfile);
 			return -1;
 		}
 	}
