@@ -100,6 +100,8 @@ int main(int argc, char **argv)
 	else
 	{
 		ret = mpg123_param(m, MPG123_VERBOSE, param.verbose, 0.);
+		if(ret == MPG123_OK)
+			mpg123_param(m, MPG123_RESYNC_LIMIT, -1, 0.0);
 		if(ret == MPG123_OK)
 		{
 			if(param.verbose)
@@ -131,6 +133,8 @@ int do_work(mpg123_handle *m)
 {
 	int ret;
 	size_t count = 0;
+	compat_binmode(STDIN_FILENO, TRUE);
+	compat_binmode(STDOUT_FILENO, TRUE);
 	ret = mpg123_open_fd(m, STDIN_FILENO);
 	if(ret != MPG123_OK) return ret;
 
