@@ -25,6 +25,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
+#if defined (IPV6) && defined (__OS2__)
+#include <libcx/net.h>
+#endif
 #endif
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
